@@ -1,34 +1,40 @@
-#!/usr/bin/python
 # -*- coding: utf-8; Mode: Python; indent-tabs-mode: nil; tab-width: 4 -*-
 
+from collections import namedtuple
+import contextlib
+import grp
 import os
 import pwd
-import grp
 import re
+import shutil
 import subprocess
 import syslog
-import shutil
-import contextlib
 
 from ubiquity import osextras
 
+
+def utf8(s, errors="strict"):
+    """Decode a string as UTF-8 if it isn't already Unicode."""
+    if isinstance(s, str):
+        return s
+    else:
+        return str(s, "utf-8", errors)
+
+
 def is_swap(device):
-    swap = False
-    fp = None
     try:
-        fp = open('/proc/swaps')
-        for line in fp:
-            if line.startswith(device + ' '):
-                swap = True
-    except:
-        swap = False
-    finally:
-        if fp:
-            fp.close()
-    return swap
+        with open('/proc/swaps') as fp:
+            for line in fp:
+                if line.startswith(device + ' '):
+                    return True
+    except Exception:
+        pass
+    return False
+
 
 _dropped_privileges = 0
 
+
 def set_groups_for_uid(uid):
     if uid == os.geteuid() or uid == os.getuid():
         return
@@ -40,6 +46,7 @@ def set_groups_for_uid(uid):
         for line in traceback.format_exc().split('\n'):
             syslog.syslog(syslog.LOG_ERR, line)
 
+
 def drop_all_privileges():
     # gconf needs both the UID and effective UID set.
     global _dropped_privileges
@@ -55,8 +62,10 @@ def drop_all_privileges():
         uid = int(uid)
         os.setreuid(uid, uid)
         os.environ['HOME'] = pwd.getpwuid(uid).pw_dir
+        os.environ['LOGNAME'] = pwd.getpwuid(uid).pw_name
     _dropped_privileges = None
 
+
 def drop_privileges():
     global _dropped_privileges
     assert _dropped_privileges is not None
@@ -73,6 +82,7 @@ def drop_privileges():
             os.seteuid(uid)
     _dropped_privileges += 1
 
+
 def regain_privileges():
     global _dropped_privileges
     assert _dropped_privileges is not None
@@ -82,6 +92,7 @@ def regain_privileges():
         os.setegid(0)
         os.setgroups([])
 
+
 def drop_privileges_save():
     """Drop the real UID/GID as well, and hide them in saved IDs."""
     # At the moment, we only know how to handle this when effective
@@ -94,17 +105,19 @@ def drop_privileges_save():
         set_groups_for_uid(uid)
     if gid is not None:
         gid = int(gid)
-        osextras.setresgid(gid, gid, 0)
+        os.setresgid(gid, gid, 0)
     if uid is not None:
-        osextras.setresuid(uid, uid, 0)
+        os.setresuid(uid, uid, 0)
+
 
 def regain_privileges_save():
     """Recover our real UID/GID after calling drop_privileges_save."""
     assert _dropped_privileges is not None and _dropped_privileges > 0
-    osextras.setresuid(0, 0, 0)
-    osextras.setresgid(0, 0, 0)
+    os.setresuid(0, 0, 0)
+    os.setresgid(0, 0, 0)
     os.setgroups([])
 
+
 @contextlib.contextmanager
 def raised_privileges():
     """As regain_privileges/drop_privileges, but in context manager style."""
@@ -114,6 +127,7 @@ def raised_privileges():
     finally:
         drop_privileges()
 
+
 def raise_privileges(func):
     """As raised_privileges, but as a function decorator."""
     from functools import wraps
@@ -125,6 +139,7 @@ def raise_privileges(func):
 
     return helper
 
+
 @raise_privileges
 def grub_options():
     """ Generates a list of suitable targets for grub-installer
@@ -134,8 +149,9 @@ def grub_options():
     l = []
     try:
         oslist = {}
-        subp = subprocess.Popen(['os-prober'], stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE)
+        subp = subprocess.Popen(
+            ['os-prober'], stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+            universal_newlines=True)
         result = subp.communicate()[0].splitlines()
         for res in result:
             res = res.split(':')
@@ -143,22 +159,12 @@ def grub_options():
         p = PartedServer()
         for disk in p.disks():
             p.select_disk(disk)
-            dev = ''
-            mod = ''
-            size = ''
-            try:
-                fp = open(p.device_entry('model'))
+            with open(p.device_entry('model')) as fp:
                 mod = fp.readline()
-                fp.close()
-                fp = open(p.device_entry('device'))
+            with open(p.device_entry('device')) as fp:
                 dev = fp.readline()
-                fp.close()
-                fp = open(p.device_entry('size'))
+            with open(p.device_entry('size')) as fp:
                 size = fp.readline()
-                fp.close()
-            finally:
-                if fp:
-                    fp.close()
             if dev and mod:
                 if size.isdigit():
                     size = format_size(int(size))
@@ -183,6 +189,8 @@ def grub_options():
             syslog.syslog(syslog.LOG_ERR, line)
     return l
 
+
+@raise_privileges
 def boot_device():
     from ubiquity.parted_server import PartedServer
 
@@ -208,6 +216,7 @@ def boot_device():
         return boot
     return root
 
+
 def is_removable(device):
     if device is None:
         return None
@@ -217,7 +226,7 @@ def is_removable(device):
     removable_bus = False
     subp = subprocess.Popen(['udevadm', 'info', '-q', 'property',
                              '-n', device],
-                            stdout=subprocess.PIPE)
+                            stdout=subprocess.PIPE, universal_newlines=True)
     for line in subp.communicate()[0].splitlines():
         line = line.strip()
         if line.startswith('DEVPATH='):
@@ -232,15 +241,17 @@ def is_removable(device):
             devpath = os.path.dirname(devpath)
         is_removable = removable_bus
         try:
-            if open('/sys%s/removable' % devpath).readline().strip() != '0':
-                is_removable = True
+            with open('/sys%s/removable' % devpath) as removable:
+                if removable.readline().strip() != '0':
+                    is_removable = True
         except IOError:
             pass
         if is_removable:
             try:
                 subp = subprocess.Popen(['udevadm', 'info', '-q', 'name',
                                          '-p', devpath],
-                                        stdout=subprocess.PIPE)
+                                        stdout=subprocess.PIPE,
+                                        universal_newlines=True)
                 return ('/dev/%s' %
                         subp.communicate()[0].splitlines()[0].strip())
             except Exception:
@@ -248,12 +259,13 @@ def is_removable(device):
 
     return None
 
+
 def mount_info(path):
-    """Return the filesystem name, type, and ro/rw used for a given mountpoint."""
+    """Return filesystem name, type, and ro/rw for a given mountpoint."""
     fsname = ''
     fstype = ''
     writable = ''
-    with contextlib.closing(open('/proc/mounts')) as fp:
+    with open('/proc/mounts') as fp:
         for line in fp:
             line = line.split()
             if line[1] == path:
@@ -262,11 +274,13 @@ def mount_info(path):
                 writable = line[3].split(',')[0]
     return fsname, fstype, writable
 
+
 def udevadm_info(args):
     fullargs = ['udevadm', 'info', '-q', 'property']
     fullargs.extend(args)
     udevadm = {}
-    subp = subprocess.Popen(fullargs, stdout=subprocess.PIPE)
+    subp = subprocess.Popen(
+        fullargs, stdout=subprocess.PIPE, universal_newlines=True)
     for line in subp.communicate()[0].splitlines():
         line = line.strip()
         if '=' not in line:
@@ -275,19 +289,47 @@ def udevadm_info(args):
         udevadm[name] = value
     return udevadm
 
+
 def partition_to_disk(partition):
     """Convert a partition device to its disk device, if any."""
     udevadm_part = udevadm_info(['-n', partition])
     if ('DEVPATH' not in udevadm_part or
-        udevadm_part.get('DEVTYPE') != 'partition'):
+            udevadm_part.get('DEVTYPE') != 'partition'):
         return partition
 
     disk_syspath = '/sys%s' % udevadm_part['DEVPATH'].rsplit('/', 1)[0]
     udevadm_disk = udevadm_info(['-p', disk_syspath])
     return udevadm_disk.get('DEVNAME', partition)
 
+
+def is_boot_device_removable(boot=None):
+    if boot:
+        return is_removable(boot)
+    else:
+        return is_removable(boot_device())
+
+
+def cdrom_mount_info():
+    """Return mount information for /cdrom.
+
+    This is the same as mount_info, except that the partition is converted to
+    its containing disk, and we don't care whether the mount point is
+    writable.
+    """
+    cdsrc, cdfs, _ = mount_info('/cdrom')
+    cdsrc = partition_to_disk(cdsrc)
+    return cdsrc, cdfs
+
+
 @raise_privileges
-def grub_default():
+def grub_device_map():
+    """Return the contents of the default GRUB device map."""
+    subp = subprocess.Popen(['grub-mkdevicemap', '--no-floppy', '-m', '-'],
+                            stdout=subprocess.PIPE, universal_newlines=True)
+    return subp.communicate()[0].splitlines()
+
+
+def grub_default(boot=None):
     """Return the default GRUB installation target."""
 
     # Much of this is intentionally duplicated from grub-installer, so that
@@ -295,13 +337,11 @@ def grub_default():
     # grub-installer is run.  Pursuant to that, we intentionally run this in
     # the installer root as /target might not yet be available.
 
-    bootremovable = is_removable(boot_device())
+    bootremovable = is_boot_device_removable(boot=boot)
     if bootremovable is not None:
         return bootremovable
 
-    subp = subprocess.Popen(['grub-mkdevicemap', '--no-floppy', '-m', '-'],
-                            stdout=subprocess.PIPE)
-    devices = subp.communicate()[0].splitlines()
+    devices = grub_device_map()
     target = None
     if devices:
         try:
@@ -312,15 +352,15 @@ def grub_default():
     if target is None:
         target = '(hd0)'
 
-    cdsrc, cdfs, type = mount_info('/cdrom')
-    cdsrc = partition_to_disk(cdsrc)
+    cdsrc, cdfs = cdrom_mount_info()
     try:
         # The target is usually under /dev/disk/by-id/, so string equality
         # is insufficient.
         same = os.path.samefile(cdsrc, target)
     except OSError:
         same = False
-    if (same or target == '(hd0)') and cdfs and cdfs != 'iso9660':
+    if ((same or target == '(hd0)') and
+            ((cdfs and cdfs != 'iso9660') or is_removable(cdsrc))):
         # Installing from removable media other than a CD.  Make sure that
         # we don't accidentally install GRUB to it.
         boot = boot_device()
@@ -337,22 +377,32 @@ def grub_default():
 
     return target
 
+
 _os_prober_oslist = {}
+_os_prober_osvers = {}
 _os_prober_called = False
 
-def find_in_os_prober(device):
-    '''Look for the device name in the output of os-prober.
-       Returns the friendly name of the device, or the empty string on error.'''
+
+def find_in_os_prober(device, with_version=False):
+    """Look for the device name in the output of os-prober.
+
+    Return the friendly name of the device, or the empty string on error.
+    """
     try:
-        oslist = os_prober()
+        oslist, osvers = os_prober()
         if device in oslist:
             ret = oslist[device]
         elif is_swap(device):
             ret = 'swap'
         else:
-            syslog.syslog('Device %s not found in os-prober output' % str(device))
+            syslog.syslog('Device %s not found in os-prober output' % device)
             ret = ''
-        return unicode(ret, 'utf-8', 'replace')
+        ret = utf8(ret, errors='replace')
+        ver = utf8(osvers.get(device, ''), errors='replace')
+        if with_version:
+            return ret, ver
+        else:
+            return ret
     except (KeyboardInterrupt, SystemExit):
         pass
     except:
@@ -360,28 +410,34 @@ def find_in_os_prober(device):
         syslog.syslog(syslog.LOG_ERR, "Error in find_in_os_prober:")
         for line in traceback.format_exc().split('\n'):
             syslog.syslog(syslog.LOG_ERR, line)
-    return unicode('')
+    return ''
+
 
 @raise_privileges
 def os_prober():
     global _os_prober_oslist
+    global _os_prober_osvers
     global _os_prober_called
-    
+
     if not _os_prober_called:
         _os_prober_called = True
-        subp = subprocess.Popen(['os-prober'], stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE)
+        subp = subprocess.Popen(
+            ['os-prober'], stdout=subprocess.PIPE, stderr=subprocess.PIPE,
+            universal_newlines=True)
         result = subp.communicate()[0].splitlines()
         for res in result:
             res = res.split(':')
             if res[2] == 'Ubuntu':
+                version = [v for v in re.findall('[0-9.]*', res[1]) if v][0]
                 # Get rid of the superfluous (development version) (11.04)
                 text = re.sub('\s*\(.*\).*', '', res[1])
                 _os_prober_oslist[res[0]] = text
+                _os_prober_osvers[res[0]] = version
             else:
                 # Get rid of the bootloader indication. It's not relevant here.
                 _os_prober_oslist[res[0]] = res[1].replace(' (loader)', '')
-    return _os_prober_oslist
+    return _os_prober_oslist, _os_prober_osvers
+
 
 @raise_privileges
 def remove_os_prober_cache():
@@ -389,25 +445,28 @@ def remove_os_prober_cache():
     shutil.rmtree('/var/lib/ubiquity/linux-boot-prober-cache',
                   ignore_errors=True)
 
-from collections import namedtuple
 
 def windows_startup_folder(mount_path):
     locations = [
+        # Windows 8
+        'ProgramData/Microsoft/Windows/Start Menu/Programs/StartUp',
         # Windows 7
         'ProgramData/Microsoft/Windows/Start Menu/Programs/Startup',
         # Windows XP
         'Documents and Settings/All Users/Start Menu/Programs/Startup',
         # Windows NT
         'Winnt/Profiles/All Users/Start Menu/Programs/Startup',
-                ]
+    ]
     for location in locations:
         path = os.path.join(mount_path, location)
         if os.path.exists(path):
             return path
     return ''
 
+
 ReleaseInfo = namedtuple('ReleaseInfo', 'name, version')
 
+
 def get_release():
     if get_release.release_info is None:
         try:
@@ -417,43 +476,46 @@ def get_release():
                     line = line.split()
                     if line[2] == 'LTS':
                         line[1] += ' LTS'
-                    get_release.release_info = ReleaseInfo(name=line[0], version=line[1])
+                    get_release.release_info = ReleaseInfo(
+                        name=line[0], version=line[1])
         except:
             syslog.syslog(syslog.LOG_ERR, 'Unable to determine the release.')
 
         if not get_release.release_info:
             get_release.release_info = ReleaseInfo(name='Ubuntu', version='')
     return get_release.release_info
+
 get_release.release_info = None
 
+
 def get_release_name():
     import warnings
     warnings.warn('get_release_name() is deprecated, '
                   'use get_release().name instead.',
                   category=DeprecationWarning)
-    
+
     if not get_release_name.release_name:
-        fp = None
         try:
-            fp = open('/cdrom/.disk/info')
-            line = fp.readline()
-            if line:
-                line = line.split()
-                if line[2] == 'LTS':
-                    get_release_name.release_name = ' '.join(line[:3])
-                else:
-                    get_release_name.release_name = ' '.join(line[:2])
+            with open('/cdrom/.disk/info') as fp:
+                line = fp.readline()
+                if line:
+                    line = line.split()
+                    if line[2] == 'LTS':
+                        get_release_name.release_name = ' '.join(line[:3])
+                    else:
+                        get_release_name.release_name = ' '.join(line[:2])
         except:
-            syslog.syslog(syslog.LOG_ERR,
-                "Unable to determine the distribution name from /cdrom/.disk/info")
-        finally:
-            if fp:
-                fp.close()
+            syslog.syslog(
+                syslog.LOG_ERR,
+                "Unable to determine the distribution name from "
+                "/cdrom/.disk/info")
         if not get_release_name.release_name:
             get_release_name.release_name = 'Ubuntu'
     return get_release_name.release_name
+
 get_release_name.release_name = ''
 
+
 @raise_privileges
 def get_install_medium():
     if not get_install_medium.medium:
@@ -463,12 +525,14 @@ def get_install_medium():
             else:
                 get_install_medium.medium = 'CD'
         except:
-            syslog.syslog(syslog.LOG_ERR,
-                "Unable to determine install medium.")
+            syslog.syslog(
+                syslog.LOG_ERR, "Unable to determine install medium.")
             get_install_medium.medium = 'CD'
     return get_install_medium.medium
+
 get_install_medium.medium = ''
 
+
 def execute(*args):
     """runs args* in shell mode. Output status is taken."""
 
@@ -477,7 +541,7 @@ def execute(*args):
 
     try:
         status = subprocess.call(log_args)
-    except IOError, e:
+    except IOError as e:
         syslog.syslog(syslog.LOG_ERR, ' '.join(log_args))
         syslog.syslog(syslog.LOG_ERR,
                       "OS error(%s): %s" % (e.errno, e.strerror))
@@ -489,10 +553,12 @@ def execute(*args):
         syslog.syslog(' '.join(log_args))
         return True
 
+
 @raise_privileges
 def execute_root(*args):
     return execute(*args)
 
+
 def format_size(size):
     """Format a partition size."""
     if size < 1000:
@@ -512,10 +578,12 @@ def format_size(size):
         factor = 1000 * 1000 * 1000 * 1000
     return '%.1f %s' % (float(size) / factor, unit)
 
+
 def debconf_escape(text):
     escaped = text.replace('\\', '\\\\').replace('\n', '\\n')
     return re.sub(r'(\s)', r'\\\1', escaped)
 
+
 def create_bool(text):
     if text == 'true':
         return True
@@ -524,12 +592,18 @@ def create_bool(text):
     else:
         return text
 
+
 @raise_privileges
 def dmimodel():
     model = ''
+    kwargs = {}
+    if os.geteuid() != 0:
+        # Silence annoying warnings during the test suite.
+        kwargs['stderr'] = open('/dev/null', 'w')
     try:
-        proc = subprocess.Popen(['dmidecode', '--string',
-            'system-manufacturer'], stdout=subprocess.PIPE)
+        proc = subprocess.Popen(
+            ['dmidecode', '--string', 'system-manufacturer'],
+            stdout=subprocess.PIPE, universal_newlines=True, **kwargs)
         manufacturer = proc.communicate()[0]
         if not manufacturer:
             return
@@ -546,7 +620,8 @@ def dmimodel():
             else:
                 key = 'system-product-name'
             proc = subprocess.Popen(['dmidecode', '--string', key],
-                                    stdout=subprocess.PIPE)
+                                    stdout=subprocess.PIPE,
+                                    universal_newlines=True)
             model = proc.communicate()[0]
         if 'apple' in manufacturer:
             # MacBook4,1 - strip the 4,1
@@ -558,6 +633,232 @@ def dmimodel():
             return
     except Exception:
         syslog.syslog(syslog.LOG_ERR, 'Unable to determine the model from DMI')
+    finally:
+        if 'stderr' in kwargs:
+            kwargs['stderr'].close()
     return model
 
+
+def set_indicator_keymaps(lang):
+    import xml.etree.cElementTree as ElementTree
+    from gi.repository import Xkl, GdkX11
+    # GdkX11.x11_get_default_xdisplay() segfaults if Gtk hasn't been
+    # imported; possibly finer-grained than this, but anything using this
+    # will already have imported Gtk anyway ...
+    from gi.repository import Gtk
+    from ubiquity import gsettings
+
+    # pacify pyflakes
+    Gtk
+
+    gsettings_key = ['org.gnome.libgnomekbd.keyboard', 'layouts']
+    lang = lang.split('_')[0]
+    variants = []
+
+    # Map inspired from that of gfxboot-theme-ubuntu that's itself
+    # based on console-setup's. This one has been restricted to
+    # language => keyboard layout not locale => keyboard layout as
+    # we don't actually know the exact locale
+    default_keymap = {
+        'ar': 'ara',
+        'bs': 'ba',
+        'de': 'de',
+        'el': 'gr',
+        'en': 'us',
+        'eo': 'epo',
+        'fr': 'fr_oss',
+        'gu': 'in_guj',
+        'hi': 'in',
+        'hr': 'hr',
+        'hy': 'am',
+        'ka': 'ge',
+        'kn': 'in_kan',
+        'lo': 'la',
+        'ml': 'in_mal',
+        'pa': 'in_guru',
+        'sr': 'rs',
+        'sv': 'se',
+        'ta': 'in_tam',
+        'te': 'in_tel',
+        'zh': 'cn',
+    }
+
+    def item_str(s):
+        '''Convert a zero-terminated byte array to a proper str'''
+        i = s.find(b'\x00')
+        return s[:i].decode()
+
+    def process_variant(*args):
+        if hasattr(args[2], 'name'):
+            variants.append(
+                '%s\t%s' % (item_str(args[1].name), item_str(args[2].name)))
+        else:
+            variants.append(item_str(args[1].name))
+
+    def restrict_list(variants):
+        new_variants = []
+
+        # Start by looking by an explicit default layout in the keymap
+        if lang in default_keymap:
+            if default_keymap[lang] in variants:
+                variants.remove(default_keymap[lang])
+                new_variants.append(default_keymap[lang])
+            else:
+                tab_keymap = default_keymap[lang].replace('_', '\t')
+                if tab_keymap in variants:
+                    variants.remove(tab_keymap)
+                    new_variants.append(tab_keymap)
+
+        # Prioritize the layout matching the language (if any)
+        if lang in variants:
+            variants.remove(lang)
+            new_variants.append(lang)
+
+        # Uniquify our list (just in case)
+        variants = list(set(variants))
+
+        if len(variants) > 4:
+            # We have a problem, X only supports 4
+
+            # Add as many entry as we can that are layouts without variant
+            country_variants = sorted(
+                entry for entry in variants if '\t' not in entry)
+            for entry in country_variants[:4 - len(new_variants)]:
+                new_variants.append(entry)
+                variants.remove(entry)
+
+            if len(new_variants) < 4:
+                # We can add some more
+                simple_variants = sorted(
+                    entry for entry in variants if '_' not in entry)
+                for entry in simple_variants[:4 - len(new_variants)]:
+                    new_variants.append(entry)
+                    variants.remove(entry)
+
+            if len(new_variants) < 4:
+                # Now just add anything left
+                for entry in variants[:4 - len(new_variants)]:
+                    new_variants.append(entry)
+                    variants.remove(entry)
+        else:
+            new_variants += list(variants)
+
+        # gsettings doesn't understand utf8
+        new_variants = [str(variant) for variant in new_variants]
+
+        return new_variants
+
+    def call_setxkbmap(variants):
+        kb_layouts = []
+        kb_variants = []
+
+        for entry in variants:
+            fields = entry.split('\t')
+            if len(fields) > 1:
+                kb_layouts.append(fields[0])
+                kb_variants.append(fields[1])
+            else:
+                kb_layouts.append(fields[0])
+                kb_variants.append("")
+
+        execute(
+            "setxkbmap", "-layout", ",".join(kb_layouts),
+            "-variant", ",".join(kb_variants))
+
+    iso_639_3 = ElementTree.parse('/usr/share/xml/iso-codes/iso_639_3.xml')
+    nodes = [element for element in iso_639_3.findall('iso_639_3_entry')
+             if element.get('part1_code') == lang]
+    display = GdkX11.x11_get_default_xdisplay()
+    engine = Xkl.Engine.get_instance(display)
+    if nodes:
+        configreg = Xkl.ConfigRegistry.get_instance(engine)
+        configreg.load(False)
+
+        # Apparently part2_code doesn't always work (fails with French)
+        for prop in ('part2_code', 'id', 'part1_code'):
+            code = nodes[0].get(prop)
+            if code is not None:
+                configreg.foreach_language_variant(code, process_variant, None)
+                if variants:
+                    restricted_variants = restrict_list(variants)
+                    call_setxkbmap(restricted_variants)
+                    gsettings.set_list(
+                        gsettings_key[0], gsettings_key[1],
+                        restricted_variants)
+                    break
+        else:
+            # Use the system default if no other keymaps can be determined.
+            gsettings.set_list(gsettings_key[0], gsettings_key[1], [])
+
+    engine.lock_group(0)
+
+
+NM = 'org.freedesktop.NetworkManager'
+NM_STATE_CONNECTED_GLOBAL = 70
+
+
+def get_prop(obj, iface, prop):
+    import dbus
+    try:
+        return obj.Get(iface, prop, dbus_interface=dbus.PROPERTIES_IFACE)
+    except dbus.DBusException as e:
+        if e.get_dbus_name() == 'org.freedesktop.DBus.Error.UnknownMethod':
+            return None
+        else:
+            raise
+
+
+def has_connection():
+    import dbus
+    bus = dbus.SystemBus()
+    manager = bus.get_object(NM, '/org/freedesktop/NetworkManager')
+    state = get_prop(manager, NM, 'state')
+    return state == NM_STATE_CONNECTED_GLOBAL
+
+
+def add_connection_watch(func):
+    import dbus
+
+    def connection_cb(state):
+        func(state == NM_STATE_CONNECTED_GLOBAL)
+
+    bus = dbus.SystemBus()
+    bus.add_signal_receiver(connection_cb, 'StateChanged', NM, NM)
+    try:
+        func(has_connection())
+    except dbus.DBusException:
+        # We can't talk to NM, so no idea.  Wild guess: we're connected
+        # using ssh with X forwarding, and are therefore connected.  This
+        # allows us to proceed with a minimum of complaint.
+        func(True)
+
+
+def install_size():
+    if min_install_size:
+        return min_install_size
+
+    # Fallback size to 5 GB
+    size = 5 * 1024 * 1024 * 1024
+
+    # Maximal size to 8 GB
+    max_size = 8 * 1024 * 1024 * 1024
+
+    try:
+        with open('/cdrom/casper/filesystem.size') as fp:
+            size = int(fp.readline())
+    except IOError:
+        pass
+
+    # TODO substitute into the template for the state box.
+    min_disk_size = size * 2  # fudge factor
+
+    # Set minimum size to 8GB if current minimum size is larger
+    # than 8GB and we still have an extra 20% of free space
+    if min_disk_size > max_size and size * 1.2 < max_size:
+        min_disk_size = max_size
+
+    return min_disk_size
+
+min_install_size = None
+
 # vim:ai:et:sts=4:tw=80:sw=4:
