@@ -1,12 +1,11 @@
-import subprocess
 import string
+import subprocess
 
 import dbus
 from dbus.mainloop.glib import DBusGMainLoop
 DBusGMainLoop(set_as_default=True)
-from gi.repository import Gtk, GObject
+from gi.repository import Gtk, GObject, GLib
 
-from ubiquity.misc import utf8
 
 NM = 'org.freedesktop.NetworkManager'
 NM_DEVICE = 'org.freedesktop.NetworkManager.Device'
@@ -25,24 +24,27 @@ NM_STATE_CONNECTED_GLOBAL = 70
 # TODO: DBus exceptions.  Catch 'em all.
 
 def decode_ssid(characters):
-    ssid = ''.join([chr(int(char)) for char in characters])
-    return utf8(ssid, errors='replace')
+    return bytearray(characters).decode('UTF-8', 'replace')
+
 
 def get_prop(obj, iface, prop):
     try:
         return obj.Get(iface, prop, dbus_interface=dbus.PROPERTIES_IFACE)
-    except dbus.DBusException, e:
+    except dbus.DBusException as e:
         if e.get_dbus_name() == 'org.freedesktop.DBus.Error.UnknownMethod':
             return None
         else:
             raise
 
+
 def get_vendor_and_model(udi):
     vendor = ''
     model = ''
     cmd = ['/sbin/udevadm', 'info', '--path=%s' % udi, '--query=property']
     with open('/dev/null', 'w') as devnull:
-        out = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=devnull)
+        out = subprocess.Popen(
+            cmd, stdout=subprocess.PIPE, stderr=devnull,
+            universal_newlines=True)
         out = out.communicate()
     if not out[1]:
         for prop in out[0].split('\n'):
@@ -52,6 +54,7 @@ def get_vendor_and_model(udi):
                 model = prop.split('ID_MODEL_FROM_DATABASE=')[1]
     return (vendor, model)
 
+
 def wireless_hardware_present():
     # NetworkManager keeps DBus objects for wireless devices around even when
     # the hardware switch is off.
@@ -67,6 +70,7 @@ def wireless_hardware_present():
             return True
     return False
 
+
 class NetworkManager:
     def __init__(self, model, state_changed=None):
         self.model = model
@@ -78,7 +82,8 @@ class NetworkManager:
 
     def start(self, state_changed=None):
         self.bus = dbus.SystemBus()
-        self.manager = self.bus.get_object(NM, '/org/freedesktop/NetworkManager')
+        self.manager = self.bus.get_object(
+            NM, '/org/freedesktop/NetworkManager')
         add = self.bus.add_signal_receiver
         add(self.queue_build_cache, 'AccessPointAdded', NM_DEVICE_WIFI, NM)
         add(self.queue_build_cache, 'AccessPointRemoved', NM_DEVICE_WIFI, NM)
@@ -125,10 +130,11 @@ class NetworkManager:
 
         obj = dbus.Dictionary(signature='sa{sv}')
         if passphrase:
-            obj['802-11-wireless-security'] = { 'psk' : passphrase }
+            obj['802-11-wireless-security'] = {'psk': passphrase}
         self.active_conn, self.active_connection = (
             self.manager.AddAndActivateConnection(
-                obj, dbus.ObjectPath(device), dbus.ObjectPath(saved_path)))
+                obj, dbus.ObjectPath(device), dbus.ObjectPath(saved_path),
+                signature='a{sa{sv}}oo'))
         self.active_device_obj = device_obj
 
     def disconnect_from_ap(self):
@@ -153,10 +159,10 @@ class NetworkManager:
                 try:
                     sec = conn_obj.GetSecrets('802-11-wireless-security',
                                               dbus_interface=NM_SETTINGS_CONN)
-                    sec = sec['802-11-wireless-security'].values()[0]
+                    sec = list(sec['802-11-wireless-security'].values())[0]
                     ssid = decode_ssid(props['802-11-wireless']['ssid'])
                     self.passphrases_cache[ssid] = sec
-                except dbus.exceptions.DBusException, e:
+                except dbus.exceptions.DBusException as e:
                     if e.get_dbus_name() != NM_ERROR_NOSECRETS:
                         raise
 
@@ -181,8 +187,8 @@ class NetworkManager:
 
     def queue_build_cache(self, *args):
         if self.timeout_id:
-            GObject.source_remove(self.timeout_id)
-        self.timeout_id = GObject.timeout_add(500, self.build_cache)
+            GLib.source_remove(self.timeout_id)
+        self.timeout_id = GLib.timeout_add(500, self.build_cache)
 
     def properties_changed(self, props, path=None):
         if 'Strength' in props:
@@ -204,7 +210,8 @@ class NetworkManager:
         devices = self.manager.GetDevices()
         for device_path in devices:
             device_obj = self.bus.get_object(NM, device_path)
-            if get_prop(device_obj, NM_DEVICE, 'DeviceType') != DEVICE_TYPE_WIFI:
+            device_type_prop = get_prop(device_obj, NM_DEVICE, 'DeviceType')
+            if device_type_prop != DEVICE_TYPE_WIFI:
                 continue
             iterator = None
             i = self.model.get_iter_first()
@@ -219,7 +226,8 @@ class NetworkManager:
                     vendor, model = get_vendor_and_model(udi)
                 else:
                     vendor, model = ('', '')
-                iterator = self.model.append(None, [device_path, vendor, model])
+                iterator = self.model.append(
+                    None, [device_path, vendor, model])
             ap_list = device_obj.GetAccessPoints(dbus_interface=NM_DEVICE_WIFI)
             ssids = []
             for ap_path in ap_list:
@@ -242,8 +250,10 @@ class NetworkManager:
         self.prune(i, devices)
         return False
 
+
 class NetworkManagerTreeView(Gtk.TreeView):
     __gtype_name__ = 'NetworkManagerTreeView'
+
     def __init__(self, password_entry=None, state_changed=None):
         Gtk.TreeView.__init__(self)
         self.password_entry = password_entry
@@ -275,17 +285,21 @@ class NetworkManagerTreeView(Gtk.TreeView):
         """
         self.expand_all()
         self.rows_changed_id = None
+
         def queue_rows_changed(*args):
             if self.rows_changed_id:
-                GObject.source_remove(self.rows_changed_id)
-            self.rows_changed_id = GObject.idle_add(self.rows_changed)
+                GLib.source_remove(self.rows_changed_id)
+            self.rows_changed_id = GLib.idle_add(self.rows_changed)
+
         model.connect('row-inserted', queue_rows_changed)
         model.connect('row-deleted', queue_rows_changed)
 
         self.user_collapsed = {}
+
         def collapsed(self, iterator, path, collapse):
             udi = model[iterator][0]
             self.user_collapsed[udi] = collapse
+
         self.connect('row-collapsed', collapsed, True)
         self.connect('row-expanded', collapsed, False)
 
@@ -397,17 +411,21 @@ class NetworkManagerTreeView(Gtk.TreeView):
         if parent:
             self.wifi_model.connect_to_ap(model[parent][0], ssid, passphrase)
 
-
 GObject.type_register(NetworkManagerTreeView)
 
+
 class NetworkManagerWidget(Gtk.Box):
     __gtype_name__ = 'NetworkManagerWidget'
-    __gsignals__ = { 'connection' : (GObject.SignalFlags.RUN_FIRST,
-                                     GObject.TYPE_NONE, (GObject.TYPE_UINT,)),
-                     'selection_changed' : (GObject.SignalFlags.RUN_FIRST,
-                                            GObject.TYPE_NONE, ()),
-                     'pw_validated' : (GObject.SignalFlags.RUN_FIRST,
-                                       GObject.TYPE_NONE, (GObject.TYPE_BOOLEAN,))}
+    __gsignals__ = {
+        'connection': (
+            GObject.SignalFlags.RUN_FIRST, GObject.TYPE_NONE,
+            (GObject.TYPE_UINT,)),
+        'selection_changed': (
+            GObject.SignalFlags.RUN_FIRST, GObject.TYPE_NONE, ()),
+        'pw_validated': (
+            GObject.SignalFlags.RUN_FIRST, GObject.TYPE_NONE,
+            (GObject.TYPE_BOOLEAN,))}
+
     def __init__(self):
         Gtk.Box.__init__(self)
         self.set_orientation(Gtk.Orientation.VERTICAL)
@@ -416,7 +434,8 @@ class NetworkManagerWidget(Gtk.Box):
         self.view = NetworkManagerTreeView(self.password_entry,
                                            self.state_changed)
         scrolled_window = Gtk.ScrolledWindow()
-        scrolled_window.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
+        scrolled_window.set_policy(
+            Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         scrolled_window.set_shadow_type(Gtk.ShadowType.IN)
         scrolled_window.add(self.view)
         self.pack_start(scrolled_window, True, True, 0)
@@ -457,12 +476,12 @@ class NetworkManagerWidget(Gtk.Box):
 
     def password_is_valid(self):
         passphrase = self.password_entry.get_text()
-        if len(passphrase) >= 8 and \
-           len(passphrase) < 64 :
+        if len(passphrase) >= 8 and len(passphrase) < 64:
             return True
         if len(passphrase) == 64:
             for c in passphrase:
-                if not c in string.hexdigits: return False
+                if not c in string.hexdigits:
+                    return False
             return True
         else:
             return False
@@ -474,7 +493,7 @@ class NetworkManagerWidget(Gtk.Box):
 
     def disconnect_from_ap(self):
         self.view.disconnect_from_ap()
-        
+
     def password_entry_changed(self, *args):
         self.emit('pw_validated', self.password_is_valid())
 
@@ -499,9 +518,9 @@ class NetworkManagerWidget(Gtk.Box):
             self.emit('pw_validated', True)
         self.emit('selection_changed')
 
-
 GObject.type_register(NetworkManagerWidget)
 
+
 if __name__ == '__main__':
     window = Gtk.Window()
     window.connect('destroy', Gtk.main_quit)
@@ -511,4 +530,3 @@ if __name__ == '__main__':
     window.add(nm)
     window.show_all()
     Gtk.main()
-
